
;**************************************************************************
;*                    IR Analyzer                                         *
;**************************************************************************
;**************************************************************************
;*          v 1.22 - 14.01.2015                                           *
;*                                                                        *
;**************************************************************************
;*          Hardw.: PIC16F1825 used                                       *
;*          OSC.......: Internal osc. used     POWER.....:   5V extern.   *
;**************************************************************************


 INCLUDE "P16F1825.INC"


_U001			     EQU     B'00100110000100'		;see Page 48 of Datasheet
_U002			     EQU     B'01100011111111'		;see Page 49 of Datasheet


	__CONFIG	_CONFIG1, _U001
	__CONFIG	_CONFIG2, _U002



;==========================================================================
;       Variable Definition
;==========================================================================
TIMER1		EQU	H'20'		;delay routine
TIMER2		EQU	H'21'		; "	"	"

v_ttt		EQU	H'22'
data_tmp	EQU	H'23'
c_tmp		EQU	H'24'
kbd_buffer	EQU	H'25'
v_tst1		EQU	H'26'
v_tst2		EQU	H'27'
v_tst3		EQU	H'28'
disp_pos	EQU	H'29'

v_ir_bit	EQU	H'2A'
v_itimer1	EQU	H'2B'
v_itimer2	EQU	H'2C'
v_ir_tmp	EQU	H'2D'
v_ir_new	EQU	H'2E'
v_ir_byte	EQU	H'2F'

v_ir_byte1	EQU	H'30'
v_ir_byte2	EQU	H'31'
v_ir_byte3	EQU	H'32'
v_ir_byte4	EQU	H'33'
v_ir_byte5	EQU	H'34'
v_ir_byte6	EQU	H'35'
v_ir_byte7	EQU	H'36'
v_ir_byte8	EQU	H'37'

v_ir_byte9	EQU	H'38'
v_ir_byte10	EQU	H'39'
v_ir_byte11	EQU	H'3A'
v_ir_byte12	EQU	H'3B'
v_ir_byte13	EQU	H'3C'
v_ir_byte14	EQU	H'3D'
v_ir_byte15	EQU	H'3E'
v_ir_byte16	EQU	H'3F'

v_ph1d_m	EQU	H'40'
v_ph1d_l	EQU	H'41'
v_ph2d_m	EQU	H'42'
v_ph2d_l	EQU	H'43'

;--------------------------------------------------------------------------
v_ir_code	EQU	H'44'		;selected IR-code (pulse width)

	;01 = Philips RC6
	;02 = Sharp
;--------------------------------------------------------------------------
v_ir_time_out	EQU	H'45'
v_irt1		EQU	H'46'
v_irt2		EQU	H'47'


;--------------------------------------------------------------------------
;==========================================================================

		ORG	0		;Reset vector address
		GOTO	init_main	;goto RESET routine when boot.

		ORG	4
		goto	ir_main		;interrupt service routine

;----------------------------------------------------------------------------
;----------------------------------------------------------------------------
;
;	       **********************************
;              **  RESET :  main boot routine  **
;              **********************************
;
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
; initialization
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------

init_main	nop

		clrf	BSR
		movlw	H'FF'
		movwf	PORTC
;
		clrf	INTCON		;disable interrupts

		banksel	OPTION_REG	;BANK 1
		movlw	B'00111111'
		movwf	OPTION_REG	;see datasheet Page 176

		movlw	H'03'
		movwf	BSR		;select BANK 3
		clrf	ANSELA
		clrf	ANSELC

		movlw	H'01'
		movwf	BSR		;select BANK 1
		movlw	B'11111110'
		movwf	TRISA

		movlw	B'11000000'
		movwf	TRISC

		movf	OSCCON,0
		andlw	B'11111111'
		iorlw	B'01111000'
		movwf	OSCCON		;set 16MHz internal clock (bit 6:3 = 1111) - Datasheet Page 68

		movlw	H'04'
		movwf	BSR		;select BANK 4
		movlw	B'00100100'
		movwf	WPUA		;enable weak pull-ups
		clrf	WPUC		;disable weak pull-ups for PORTC

		movlw	H'07'
		movwf	BSR		;select BANK 7
		movlw	H'FF'
		movwf	INLVLA		;ST (1) level or TTL (0) level (?)
		movwf	INLVLC		;ST (1) level or TTL (0) level (?)

		banksel	LATC
		clrf	LATC

		clrf	BSR		;select BANK 0

		clrf	PORTA
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
		call	disp_init
		call	w_release

		call	dr4
		call	dr4
		call	dr4
		call	dr4
		call	dr4
		call	dr4

;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
;set-up interrupt in INT-pin (RA2)
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------

		clrf	INTCON

;bit INTEDG (bit 6) of OPTION_REG needs to be cleared - is defined above

		bsf	INTCON,INTE			;Enables the INT external interrupt
		bsf	INTCON,GIE			;Enables all active interrupts
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------

		goto	main_prg


;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;------------------------------------------------------------------------------
;                      Interrupt service routine
;------------------------------------------------------------------------------
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX

ir_main		nop
		btfss	INTCON,INTF		;unkown event -- return
		goto	leave_ir

		bcf	INTCON,D'007'		;disable interrupts
		movf	PORTA,0
		bcf	INTCON,INTF

		btfsc	v_ir_new,D'000'
		goto	leave_ir		;last value not processed yet

;-----------------------------------------------------------------------------
;Interrupt business logic starts here....
;-----------------------------------------------------------------------------

		bsf	PORTA,D'000'		;switch the LED on - we got signal...
		btfsc	PORTA,D'002'
		goto	ir_cancel		;we expect level 0 now...


		call	ir_get_byte		;receive 1st byte
		movf	v_ir_byte,0
		movwf	v_ir_byte1

		call	ir_get_byte		;receive 2nd byte
		movf	v_ir_byte,0
		movwf	v_ir_byte2

		call	ir_get_byte		;receive 3rd byte
		movf	v_ir_byte,0
		movwf	v_ir_byte3

		call	ir_wait_change		;now execute the synchronization step (okro IR code 2)


		call	ir_get_byte		;receive 4th byte
		movf	v_ir_byte,0
		movwf	v_ir_byte4

		call	ir_get_byte		;receive 5th byte
		movf	v_ir_byte,0
		movwf	v_ir_byte5

		call	ir_get_byte		;receive 6th byte
		movf	v_ir_byte,0
		movwf	v_ir_byte6

		call	ir_get_byte		;receive 7th byte
		movf	v_ir_byte,0
		movwf	v_ir_byte7

		call	ir_get_byte		;receive 8th byte
		movf	v_ir_byte,0
		movwf	v_ir_byte8

		call	ir_wait_change		;now execute the synchronization step (okro IR code 2)

		call	ir_get_byte
		movf	v_ir_byte,0
		movwf	v_ir_byte9

		call	ir_get_byte
		movf	v_ir_byte,0
		movwf	v_ir_byte10

		call	ir_get_byte
		movf	v_ir_byte,0
		movwf	v_ir_byte11

		call	ir_get_byte
		movf	v_ir_byte,0
		movwf	v_ir_byte12

		call	ir_get_byte
		movf	v_ir_byte,0
		movwf	v_ir_byte13

		call	ir_get_byte
		movf	v_ir_byte,0
		movwf	v_ir_byte14

		call	ir_get_byte
		movf	v_ir_byte,0
		movwf	v_ir_byte15

		call	ir_get_byte
		movf	v_ir_byte,0
		movwf	v_ir_byte16



		clrf	v_ir_new		;store the code
		bsf	v_ir_new,D'000'		;set flag - new character received

;----------------------------------------
ir_cancel	bcf	PORTA,D'000'		;switch the LED off
;-----------------------------------------------------------------------------
;Interrupt business logic ends here....
;-----------------------------------------------------------------------------
leave_ir	nop
		bcf	PORTA,D'000'		;switch the LED off
		bsf	INTCON,INTE
		bsf	INTCON,D'007'
		RETFIE				;return from interrupt with restore of STAUS, WREG and BSR
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------


;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
;synchronization step
;wait until next change on the line happens
;exit waiting after time out - apx.1ms

ir_wait_change	clrf	v_ir_time_out
		clrf	v_irt1
		clrf	v_irt2

		btfsc	PORTA,D'002'
		goto	ir_w_1to0
ir_w_0to1	call	w_ir_inc_timer
		btfsc	v_ir_time_out,D'000'
		return				;time out
		btfss	PORTA,D'002'
		goto	ir_w_0to1
		return				;input value changed: 0 -> 1
;------------------------------------------------------------------------------
ir_w_1to0	call	w_ir_inc_timer
		btfsc	v_ir_time_out,D'000'
		return				;time out
		btfsc	PORTA,D'002'
		goto	ir_w_0to1
		return				;input value changed: 1 -> 0
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
w_ir_inc_timer	incf	v_irt1,1
		btfss	STATUS,Z
		return

		incf	v_irt2,1
		movf	v_irt2,0
		xorlw	D'005'
		btfss	STATUS,Z
		return

		bsf	v_ir_time_out,D'000'
		return
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------


;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
ir_get_byte	clrf	v_ir_byte
		call	ir_get_bit
		btfsc	v_ir_bit,D'000'
		bsf	v_ir_byte,D'007'

		call	ir_get_bit
		btfsc	v_ir_bit,D'000'
		bsf	v_ir_byte,D'006'

		call	ir_get_bit
		btfsc	v_ir_bit,D'000'
		bsf	v_ir_byte,D'005'

		call	ir_get_bit
		btfsc	v_ir_bit,D'000'
		bsf	v_ir_byte,D'004'

		call	ir_get_bit
		btfsc	v_ir_bit,D'000'
		bsf	v_ir_byte,D'003'

		call	ir_get_bit
		btfsc	v_ir_bit,D'000'
		bsf	v_ir_byte,D'002'

		call	ir_get_bit
		btfsc	v_ir_bit,D'000'
		bsf	v_ir_byte,D'001'

		call	ir_get_bit
		btfsc	v_ir_bit,D'000'
		bsf	v_ir_byte,D'000'
		return
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
ir_get_bit	clrf	v_ir_bit
		call	wait_Ph1
		btfsc	PORTA,D'002'
		bsf	v_ir_bit,D'000'
		call	wait_Ph2
		return
;------------------------------------------------------------------------------
ir_get_bit2	clrf	v_ir_bit
		call	wait300us
		btfsc	PORTA,D'002'
		bsf	v_ir_bit,D'000'
		call	wait460us
		return
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
; interrupt delay routines
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
d350		MOVLW   D'255'
		MOVWF   v_itimer2
iDEL_LOOP1	MOVLW   D'255'
		MOVWF   v_itimer1

iDEL_LOOP2	DECFSZ  v_itimer1,F
		GOTO    iDEL_LOOP2
		DECFSZ  v_itimer2,F
		GOTO    iDEL_LOOP1
		return
;----------------------------------------------------------------------------
;----------------------------------------------------------------------------
;Phase 1 wait routine
;----------------------------------------------------------------------------
;----------------------------------------------------------------------------
wait_Ph1	movf	v_ph1d_m,0
		movwf	v_itimer2
iw2_l2		movf	v_ph1d_l,0
		movwf	v_itimer1

iw2_l1		decfsz	v_itimer1,1		;0,25us
		goto	iw2_l1			;0,50us

		decfsz	v_itimer2,1
		goto	iw2_l2
		return

;----------------------------------------------------------------------------
;----------------------------------------------------------------------------
;Phase 2 wait routine
;----------------------------------------------------------------------------
;----------------------------------------------------------------------------
wait_Ph2	movf	v_ph2d_m,0
		movwf	v_itimer2
iw28_l2		movf	v_ph2d_l,0
		movwf	v_itimer1

iw28_l1		decfsz	v_itimer1,1		;0,25us
		goto	iw28_l1			;0,50us (16MHz clock)

		decfsz	v_itimer2,1
		goto	iw28_l2
		return

;----------------------------------------------------------------------------
;----------------------------------------------------------------------------
;----------------------------------------------------------------------------
wait300us	movlw	D'002'
		movwf	v_itimer2
iw3_l2		movlw	D'210'
		movwf	v_itimer1

iw3_l1		decfsz	v_itimer1,1		;0,5us / 2
		goto	iw3_l1			;1,0us / 2

		decfsz	v_itimer2,1
		goto	iw3_l2
		return

;----------------------------------------------------------------------------
wait460us	movlw	D'004'
		movwf	v_itimer2
iw4_l2		movlw	D'155'
		movwf	v_itimer1

iw4_l1		decfsz	v_itimer1,1		;0,5us / 2
		goto	iw4_l1			;1,0us / 2

		decfsz	v_itimer2,1
		goto	iw4_l2
		return

;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;------------------------------------------------------------------------------
;                      Waiting times definitions
;------------------------------------------------------------------------------
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
;----------------------------------------------------

;----------------------------------------------------
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;----------------------------------------------------
;delay values for Philips code RC6
;PIC Clock = 16MHz
;in total: 444us
;
;Phase 1 in total ~ 1 * 245 * 0,75us = 183,75 us
;Phase 2 in total ~ 2 * 169 * 0,75us = 253,5 us
;----------------------------------------------------
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;----------------------------------------------------

wt1_philips1	movlw	D'001'
		movwf	v_ph1d_m
		movlw	D'245'
		movwf	v_ph1d_l

		movlw	D'002'
		movwf	v_ph2d_m
		movlw	D'169'
		movwf	v_ph2d_l

		return

;----------------------------------------------------
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;----------------------------------------------------
;delay values for Sharp
;PIC Clock = 16MHz
;in total: 300us
;
;Phase 1 in total ~ 1 * 201 * 0,75us = 150,75 us
;Phase 2 in total ~ 1 * 200 * 0,75us = 150,00 us
;----------------------------------------------------
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;----------------------------------------------------

wt2_sharp1	movlw	D'001'
		movwf	v_ph1d_m
		movlw	D'201'
		movwf	v_ph1d_l

		movlw	D'001'
		movwf	v_ph2d_m
		movlw	D'200'
		movwf	v_ph2d_l

		return

;----------------------------------------------------
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;----------------------------------------------------
;delay values for Sony
;PIC Clock = 16MHz
;in total: 600us
;
;Phase 1 in total ~ 2 * 200 * 0,75us = 300,0 us
;Phase 2 in total ~ 2 * 200 * 0,75us = 300,0 us
;----------------------------------------------------
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;----------------------------------------------------

wt3_sony1	movlw	D'002'
		movwf	v_ph1d_m
		movlw	D'200'
		movwf	v_ph1d_l


		movlw	D'002'
		movwf	v_ph2d_m
		movlw	D'200'
		movwf	v_ph2d_l

		return


;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;----------------------------------------------------
;delay values for Panasonic
;PIC Clock = 16MHz
;in total: 423us
;
;Phase 1 in total ~ 3 * 141 * 0,75us = 317,25 us
;Phase 2 in total ~ 1 * 141 * 0,75us = 105,75 us
;----------------------------------------------------
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;----------------------------------------------------

wt4_pan1	movlw	D'003'
		movwf	v_ph1d_m
		movlw	D'141'
		movwf	v_ph1d_l

		movlw	D'001'
		movwf	v_ph2d_m
		movlw	D'141'
		movwf	v_ph2d_l

		return

;----------------------------------------------------
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;----------------------------------------------------
;delay values for Aiwa
;PIC Clock = 16MHz
;in total: 640us
;
;Phase 1 in total ~ 1 * 241 * 0,75us = 180,75 us
;Phase 2 in total ~ 3 * 203 * 0,75us = 456,75 us
;----------------------------------------------------
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;----------------------------------------------------

wt5_aiwa1	movlw	D'001'
		movwf	v_ph1d_m
		movlw	D'241'
		movwf	v_ph1d_l


		movlw	D'003'
		movwf	v_ph2d_m
		movlw	D'203'
		movwf	v_ph2d_l

		return

;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
select_times	movf	v_ir_code,0
		xorlw	D'001'
		btfsc	STATUS,Z
		goto	wt1_philips1			;set-up times for Philips RC6 code

		movf	v_ir_code,0
		xorlw	D'002'
		btfsc	STATUS,Z
		goto	wt2_sharp1			;set-up times for "Sharp code"

		movf	v_ir_code,0
		xorlw	D'003'
		btfsc	STATUS,Z
		goto	wt3_sony1			;Sony

		movf	v_ir_code,0
		xorlw	D'004'
		btfsc	STATUS,Z
		goto	wt4_pan1			;Panasonic

		movf	v_ir_code,0
		xorlw	D'005'
		btfsc	STATUS,Z
		goto	wt5_aiwa1			;Aiwa


		goto	wt1_philips1			;default...
		return


;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;------------------------------------------------------------------------------
;                      Main Loop
;------------------------------------------------------------------------------
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------

main_prg	movlw	H'FF'
		movwf	v_ir_byte1
		movwf	v_ir_byte9
		movwf	v_ir_new


		call	clear_disp


		movlw	D'001'
		movwf	v_ir_code		;select Philips RC6 IR-code
		call	select_times


main_prg2	call	w_status
		clrf	v_ir_new

mp_wait		btfss	PORTA,D'005'
		call	change_code
		btfss	v_ir_new,D'000'
		goto	mp_wait

		goto	main_prg2
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
change_code	incf	v_ir_code,1
		movf	v_ir_code,0
		xorlw	D'006'			;there are exactly 3 codes implemented now
		btfss	STATUS,Z
		goto	cc_ret

		movlw	D'001'
		movwf	v_ir_code
;----------------------------------------------
cc_ret		call	select_times
		call	w_status

		movlw	H'FF'
		movwf	v_ir_byte1
		movwf	v_ir_byte9
		movwf	v_ir_new

cc_ret2		btfss	PORTA,D'005'
		goto	cc_ret2			;wait until the button has been released

		return

;------------------------------------------------------------------------------
;------------------------------------------------------------------------------

;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX


;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
; delay routines
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
d2		MOVLW   D'255'
		MOVWF   TIMER2
DEL_LOOP1	MOVLW   D'255'
		MOVWF   TIMER1

DEL_LOOP2	DECFSZ  TIMER1,F
		GOTO    DEL_LOOP2
		DECFSZ  TIMER2,F
		GOTO    DEL_LOOP1
		return
;------------------------------------------------------------------------------
d2c		MOVLW   D'090'
		MOVWF   TIMER2
cDEL_LOOP1	MOVLW   D'255'
		MOVWF   TIMER1

cDEL_LOOP2	DECFSZ  TIMER1,F
		GOTO    cDEL_LOOP2
		DECFSZ  TIMER2,F
		GOTO    cDEL_LOOP1
		return
;------------------------------------------------------------------------------

;------------------------------------------------------------------------------
;------------------------------------------------------------------------------

;------------------------------------
dr2		call	d2
		call	d2
		call	d2
		return
;------------------------------------
dr4		call	dr2
		call	dr2
		call	dr2
		return


;xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
;x           LCD Display Communication Routines                               x
;xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx

;RS  = RE2 -- RA5 -- RC4
;R/W = RE1 -- N/C -- N/C
;E   = RE0 -- RA4 -- RC5

;D0 - D3 -- not connected

;D4  = RD0 -- RA0 -- RC0
;D5  = RD1 -- RA1 -- RC1
;D6  = RD2 -- RA2 -- RC2
;D7  = RD3 -- RA3 -- RC3


disp_init	nop

;1st step of init:
;		wait for LCD hardware reset (after power on) - no software reset performed...
		clrf	PORTC
		call	d2_p2

;-- SW Reset to be able to enter 4-bit mode
		bcf	PORTC,D'004'
		bcf	PORTC,D'005'		;EN = 0 => disable


		movlw	B'00000011'
		call	set_LCD_D
		bsf	PORTC,D'00'
		call	dy_comm
		bcf	PORTC,D'005'
		call	d2_p2

		movlw	B'00000011'
		call	set_LCD_D
		bsf	PORTC,D'005'
		call	dy_comm
		bcf	PORTC,D'005'
		call	d2_p2

		movlw	B'00000011'
		call	set_LCD_D
		bsf	PORTC,D'005'
		call	dy_comm
		bcf	PORTC,D'005'
		call	d2_p2

		movlw	B'00000010'
		call	set_LCD_D
		bsf	PORTC,D'005'
		call	dy_comm
		bcf	PORTC,D'005'
		call	d2_p2

;-----------------------------------------------------------------
;2nd step of init:
;     command:           '001DNFxx' (SET FUNCTION)
;                            D = 1	--> 8 bit interface
;                            D = 0	--> 4 bit interface
;			 '001DNFxx'
;                             N = 1	--> 2 line display
;                             N = 0	--> 1 line display
;			 '001DNFxx'
;                              F = 1	--> 5x10
;			       F = 0	--> 5x7
;			 '001DNFxx'
		movlw	B'00101000'		;2 lines - (for disp 1x16: 1-8  = 00h - 0Fh
;                                                                          9-16 = 40h - 4Fh )
		call	send_comm
;-----------------------------------------------------------------
;3rd step of init:
;     command:           '00001DCB' (DISPLAY ON/OFF)
;                              D = 1	--> Display on
;			       D = 0	--> Display off
;           		 '00001DCB'
;                               C = 1	--> Cursor on
;				C = 0	--> Cursor off
;           		 '00001DCB'
;				 B = 1	--> Blinking on
;				 B = 0	--> Blinking off
		movlw	B'00001000'
		call	send_comm

		movlw	B'00001111'
		movlw	B'00001100'
		call	send_comm
;-----------------------------------------------------------------
;4th step of init:
;     command:           '000001DS' (ENTRY MODE SET)
;				D = 1	--> Increase
;				D = 0	--> Decrease
;		         '000001DS'
;				 S = 1	--> display is shifted
;				 S = 0	--> display is not shifted
;		         '000001DS'
		movlw	B'00000110'
		call	send_comm
;-----------------------------------------------------------------
;clear the display
		movlw	B'00000001'
		call	send_comm
		return
;------------------------------------------------------------------------------------------------
;------------------------------------------------------------------------------------------------
;------------------------------------------------------------------------------------------------

set_LCD_D	nop
		movwf	v_ttt

		btfss	v_ttt,D'000'
		bcf	PORTC,D'000'
		call	set_0

		btfss	v_ttt,D'001'
		bcf	PORTC,D'001'
		call	set_1

		btfss	v_ttt,D'002'
		bcf	PORTC,D'002'
		call	set_2

		btfss	v_ttt,D'003'
		bcf	PORTC,D'003'
		call	set_3
		return
;-------------------------------------------
set_0		btfsc	v_ttt,D'000'
		bsf	PORTC,D'000'
		return

set_1		btfsc	v_ttt,D'001'
		bsf	PORTC,D'001'
		return

set_2		btfsc	v_ttt,D'002'
		bsf	PORTC,D'002'
		return

set_3		btfsc	v_ttt,D'003'
		bsf	PORTC,D'003'
		return
;------------------------------------------------------------------------------------------------
;8-bit data transfer routines
;------------------------------------------------------------------------------------------------
send_data	bcf	PORTC,D'005'		;EN -> 0
		movwf	data_tmp

		swapf	data_tmp,0
		andlw	B'00001111'
		call	set_LCD_D		;higher 4 bits are put to RA0-RA3
		call	scbd			;send

		movf	data_tmp,0
		andlw	B'00001111'
		call	set_LCD_D		;lower 4 bits are put to RA0-RA3
		call	scbd			;send

		return

;set control bits
scbd		bsf	PORTC,D'004'		;RS -> 1 (data)
		bsf	PORTC,D'005'		;EN -> 1
		call	dy_comm			;wait a while...
		bcf	PORTC,D'005'		;EN -> 0
		call	d1			;wait a little bit...

		return
;------------------------------------------------------------------------------------------------
send_comm	bcf	PORTC,D'005'		;EN -> 0
		movwf	data_tmp

		swapf	data_tmp,0
		andlw	B'00001111'
		call	set_LCD_D		;higher 4 bits are put to RA0-RA3
		call	scbc			;send

		movf	data_tmp,0
		andlw	B'00001111'
		call	set_LCD_D		;lower 4 bits are put to RA0-RA3
		call	scbc			;send

		return

;set control bits
scbc		bcf	PORTC,D'004'		;RS -> 0 (command)
		bsf	PORTC,D'005'		;EN -> 1
		call	dy_comm			;wait a while...
		bcf	PORTC,D'005'		;EN -> 0
		call	d1			;wait a little bit...
		return
;-----------------------------------------------------------------------------------------------
;----------------------------------------------
dy_comm		call	dy_comm2	;01
		call	dy_comm2	;02
		call	dy_comm2	;03
		return

dy_comm2	MOVLW   D'030'			;delay value changed: original value was: 010
		MOVWF   TIMER2
DEL_LOOP22c	DECFSZ  TIMER2,F
                GOTO    DEL_LOOP22c
		return
;---------------------------------------------------------------------------
;
d1		call	d1b		;01
		call	d1b		;02
		call	d1b		;03
		call	d1b		;04
		call	d1b		;05
		call	d1b		;06
		call	d1b		;07
		call	d1b		;08
		call	d1b		;09
		return

d1b		MOVLW   D'255'
		MOVWF   TIMER2
DEL_LOOP11	DECFSZ  TIMER2,F
                GOTO    DEL_LOOP11
		return
;
d2_p2		MOVLW   D'255'
		MOVWF   TIMER2
DEL_LOOPx1	MOVLW   D'255'
		MOVWF   TIMER1

DEL_LOOPx2	DECFSZ  TIMER1,F
		GOTO    DEL_LOOPx2
		DECFSZ  TIMER2,F
		GOTO    DEL_LOOPx1
		return

;---------------------------------------------------------------------------
;---------------------------------------------------------------------------
;------------------------------------------------------------------------------------------------
clear_disp	call	disp_clr1
		call	disp_clr2
		return
;------------------------------------------------------------------------------------------------
;------------------------------------------------------------------------------------------------
n9_char		movlw	B'11000000'		;od znaku c.9 treba zmenit adresu na 40h
		call	send_comm

		movlw	B'10000000'
		movwf	disp_pos
		return
;------------------------------------------------------------------------------------------------
;------------------------------------------------------------------------------------------------
init2		movlw	B'00000001'
;		call	send_comm
		movlw	B'10000000'
		call	send_comm
		return

;xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
;xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
;
;character set
;
;xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
;xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx

wm_space	movlw	' '
		call	send_data
		return

wm_dvojbodka	movlw	':'
		call	send_data
		return

wm_point	nop
wm_bodka	movlw	'.'
		call	send_data
		return

wm_hviezdicka	movlw	'*'
		call	send_data
		return

wm_otaznik	movlw	'?'
		call	send_data
		return

wm_sls		movlw	'/'
		call	send_data
		return

wm_kk		movlw	'#'
		call	send_data
		return

wm_minus	movlw	'-'
		call	send_data
		return

wm_plus		movlw	'+'
		call	send_data
		return

wm_hvizdicka	movlw	'*'
		call	send_data
		return

wm_zat1		movlw	'('
		call	send_data
		return

wm_zat2		movlw	')'
		call	send_data
		return

wm_sip1		movlw	'>'
		call	send_data
		return

wm_sip2		movlw	'<'
		call	send_data
		return

wm_stup		movlw	H'DF'
		call	send_data
		return

wm_rs		movlw	'='
		call	send_data
		return

wm_and		movlw	'&'
		call	send_data
		return
;---------- cislice ----------------------------------------
wm_0		movlw	'0'
		call	send_data
		return

wm_1		movlw	'1'
		call	send_data
		return

wm_2		movlw	'2'
		call	send_data
		return

wm_3		movlw	'3'
		call	send_data
		return

wm_4		movlw	'4'
		call	send_data
		return

wm_5		movlw	'5'
		call	send_data
		return

wm_6		movlw	'6'
		call	send_data
		return

wm_7		movlw	'7'
		call	send_data
		return

wm_8		movlw	'8'
		call	send_data
		return

wm_9		movlw	'9'
		call	send_data
		return

;--------- velke pismena -----------------------------------
wm_A		movlw	'A'
		call	send_data
		return

wm_B		movlw	'B'
		call	send_data
		return

wm_C		movlw	'C'
		call	send_data
		return

wm_D		movlw	'D'
		call	send_data
		return

wm_E		movlw	'E'
		call	send_data
		return

wm_F		movlw	'F'
		call	send_data
		return

wm_G		movlw	'G'
		call	send_data
		return

wm_H		movlw	'H'
		call	send_data
		return

wm_I		movlw	'I'
		call	send_data
		return

wm_J		movlw	'J'
		call	send_data
		return

wm_K		movlw	'K'
		call	send_data
		return

wm_L		movlw	'L'
		call	send_data
		return

wm_M		movlw	'M'
		call	send_data
		return

wm_N		movlw	'N'
		call	send_data
		return

wm_O		movlw	'O'
		call	send_data
		return

wm_P		movlw	'P'
		call	send_data
		return

wm_Q		movlw	'Q'
		call	send_data
		return

wm_R		movlw	'R'
		call	send_data
		return

wm_S		movlw	'S'
		call	send_data
		return

wm_T		movlw	'T'
		call	send_data
		return

wm_U		movlw	'U'
		call	send_data
		return

wm_V		movlw	'V'
		call	send_data
		return

wm_W		movlw	'W'
		call	send_data
		return

wm_X		movlw	'X'
		call	send_data
		return

wm_Y		movlw	'Y'
		call	send_data
		return

wm_Z		movlw	'Z'
		call	send_data
		return

;--------- male pismena -----------------------------------

wm_a		movlw	'a'
		call	send_data
		return

wm_b		movlw	'b'
		call	send_data
		return

wm_c		movlw	'c'
		call	send_data
		return

wm_d		movlw	'd'
		call	send_data
		return

wm_e		movlw	'e'
		call	send_data
		return

wm_f		movlw	'f'
		call	send_data
		return

wm_g		movlw	'g'
		call	send_data
		return

wm_h		movlw	'h'
		call	send_data
		return

wm_i		movlw	'i'
		call	send_data
		return

wm_k		movlw	'k'
		call	send_data
		return

wm_l		movlw	'l'
		call	send_data
		return

wm_m		movlw	'm'
		call	send_data
		return

wm_n		movlw	'n'
		call	send_data
		return

wm_o		movlw	'o'
		call	send_data
		return

wm_p		movlw	'p'
		call	send_data
		return

wm_r		movlw	'r'
		call	send_data
		return

wm_s		movlw	's'
		call	send_data
		return

wm_t		movlw	't'
		call	send_data
		return

wm_u		movlw	'u'
		call	send_data
		return

wm_v		movlw	'v'
		call	send_data
		return

wm_w		movlw	'w'
		call	send_data
		return

wm_x		movlw	'x'
		call	send_data
		return

wm_y		movlw	'y'
		call	send_data
		return
;xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
;xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
;xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
;xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx


;---------------------------------------------------------------------------
disp_clr1	movlw	B'10000000'		;line 1 / char0
		call	send_comm

disp_clrx	call	wm_space	;01
		call	wm_space	;02
		call	wm_space	;03
		call	wm_space	;04
		call	wm_space	;05
		call	wm_space	;06
		call	wm_space	;07
		call	wm_space	;08
		call	wm_space	;09
		call	wm_space	;10
		call	wm_space	;11
		call	wm_space	;12
		call	wm_space	;13
		call	wm_space	;14
		call	wm_space	;15
		call	wm_space	;16
		call	wm_space	;17
		call	wm_space	;18
		call	wm_space	;19
		call	wm_space	;20
		call	wm_space	;21
		call	wm_space	;22
		call	wm_space	;23
		call	wm_space	;24

		return
;---------------------------------------------------------------------------
disp_clr2	movlw	B'11000000'		;line 2 / char0
		call	send_comm

		goto	disp_clrx

;-----------------------------------------------------------------------------
;-----------------------------------------------------------------------------
convert2ascii	movwf	c_tmp
;
		xorlw	H'15'
		btfsc	STATUS,Z
		goto	wm_Q
;
		movf	c_tmp,0
		xorlw	H'1D'
		btfsc	STATUS,Z
		goto	wm_W
;
		movf	c_tmp,0
		xorlw	H'24'
		btfsc	STATUS,Z
		goto	wm_E
;
		movf	c_tmp,0
		xorlw	H'2D'
		btfsc	STATUS,Z
		goto	wm_R
;
		movf	c_tmp,0
		xorlw	H'2C'
		btfsc	STATUS,Z
		goto	wm_T
;
		movf	c_tmp,0
		xorlw	H'35'
		btfsc	STATUS,Z
		goto	wm_Y
;
		movf	c_tmp,0
		xorlw	H'3C'
		btfsc	STATUS,Z
		goto	wm_U
;
		movf	c_tmp,0
		xorlw	H'43'
		btfsc	STATUS,Z
		goto	wm_I
;
		movf	c_tmp,0
		xorlw	H'44'
		btfsc	STATUS,Z
		goto	wm_O
;
		movf	c_tmp,0
		xorlw	H'4D'
		btfsc	STATUS,Z
		goto	wm_P
;
		movf	c_tmp,0
		xorlw	H'16'
		btfsc	STATUS,Z
		goto	wm_1
;
		movf	c_tmp,0
		xorlw	H'1E'
		btfsc	STATUS,Z
		goto	wm_2
;
		movf	c_tmp,0
		xorlw	H'26'
		btfsc	STATUS,Z
		goto	wm_3
;
		movf	c_tmp,0
		xorlw	H'25'
		btfsc	STATUS,Z
		goto	wm_4
;
		movf	c_tmp,0
		xorlw	H'2E'
		btfsc	STATUS,Z
		goto	wm_5
;
		movf	c_tmp,0
		xorlw	H'36'
		btfsc	STATUS,Z
		goto	wm_6
;
		movf	c_tmp,0
		xorlw	H'3D'
		btfsc	STATUS,Z
		goto	wm_7
;
		movf	c_tmp,0
		xorlw	H'3E'
		btfsc	STATUS,Z
		goto	wm_8
;
		movf	c_tmp,0
		xorlw	H'46'
		btfsc	STATUS,Z
		goto	wm_9
;
		movf	c_tmp,0
		xorlw	H'45'
		btfsc	STATUS,Z
		goto	wm_0
;
		movf	c_tmp,0
		xorlw	H'1C'
		btfsc	STATUS,Z
		goto	wm_A
;
		movf	c_tmp,0
		xorlw	H'1B'
		btfsc	STATUS,Z
		goto	wm_S
;
		movf	c_tmp,0
		xorlw	H'23'
		btfsc	STATUS,Z
		goto	wm_D
;
		movf	c_tmp,0
		xorlw	H'2B'
		btfsc	STATUS,Z
		goto	wm_F
;
		movf	c_tmp,0
		xorlw	H'34'
		btfsc	STATUS,Z
		goto	wm_G
;
		movf	c_tmp,0
		xorlw	H'33'
		btfsc	STATUS,Z
		goto	wm_H
;
		movf	c_tmp,0
		xorlw	H'3B'
		btfsc	STATUS,Z
		goto	wm_J
;
		movf	c_tmp,0
		xorlw	H'42'
		btfsc	STATUS,Z
		goto	wm_K
;
		movf	c_tmp,0
		xorlw	H'4B'
		btfsc	STATUS,Z
		goto	wm_L
;
		movf	c_tmp,0
		xorlw	H'1A'
		btfsc	STATUS,Z
		goto	wm_Z
;
		movf	c_tmp,0
		xorlw	H'22'
		btfsc	STATUS,Z
		goto	wm_X
;
		movf	c_tmp,0
		xorlw	H'21'
		btfsc	STATUS,Z
		goto	wm_C
;
		movf	c_tmp,0
		xorlw	H'2A'
		btfsc	STATUS,Z
		goto	wm_V
;
		movf	c_tmp,0
		xorlw	H'32'
		btfsc	STATUS,Z
		goto	wm_B
;
		movf	c_tmp,0
		xorlw	H'31'
		btfsc	STATUS,Z
		goto	wm_N
;
		movf	c_tmp,0
		xorlw	H'3A'
		btfsc	STATUS,Z
		goto	wm_M
;
		movf	c_tmp,0
		xorlw	H'29'
		btfsc	STATUS,Z
		goto	wm_space
;
		movf	c_tmp,0
		xorlw	H'6C'
		btfsc	STATUS,Z
		goto	wm_7
;
		movf	c_tmp,0
		xorlw	H'75'
		btfsc	STATUS,Z
		goto	wm_8
;
		movf	c_tmp,0
		xorlw	H'7D'
		btfsc	STATUS,Z
		goto	wm_9
;
		movf	c_tmp,0
		xorlw	H'6B'
		btfsc	STATUS,Z
		goto	wm_4
;
		movf	c_tmp,0
		xorlw	H'73'
		btfsc	STATUS,Z
		goto	wm_5
;
		movf	c_tmp,0
		xorlw	H'74'
		btfsc	STATUS,Z
		goto	wm_6
;
		movf	c_tmp,0
		xorlw	H'69'
		btfsc	STATUS,Z
		goto	wm_1
;
		movf	c_tmp,0
		xorlw	H'72'
		btfsc	STATUS,Z
		goto	wm_2
;
		movf	c_tmp,0
		xorlw	H'7A'
		btfsc	STATUS,Z
		goto	wm_3
;
		movf	c_tmp,0
		xorlw	H'70'
		btfsc	STATUS,Z
		goto	wm_0
;
		movf	c_tmp,0
		xorlw	H'71'
		btfsc	STATUS,Z
		goto	wm_bodka
;
		movf	c_tmp,0
		xorlw	H'79'
		btfsc	STATUS,Z
		goto	wm_plus
;
		movf	c_tmp,0
		xorlw	H'7B'
		btfsc	STATUS,Z
		goto	wm_minus
;
		movf	c_tmp,0
		xorlw	H'7C'
		btfsc	STATUS,Z
		goto	wm_hvizdicka

		movf	c_tmp,0
		xorlw	H'76'			;ESC
		btfsc	STATUS,Z
;!!!!!!		goto	main_s


;not known character - either transmission error or not decoded... - do nothing...
		decf	disp_pos,1
		
		return


		return
;-------------------------------------------------------------------------------
;-------------------------------------------------------------------------------
;-------------------------------------------------------------------------------
w_wt1		call	wm_P
		call	wm_h
		call	wm_i
		call	wm_l
		call	wm_i
		call	wm_p
		call	wm_s
		call	wm_space
		call	wm_space
		call	wm_space
		return
;----------------------------------
w_wt2		call	wm_S
		call	wm_h
		call	wm_a
		call	wm_r
		call	wm_p
		call	wm_space
		call	wm_space
		call	wm_space
		call	wm_space
		call	wm_space
		return
;----------------------------------
w_wt3		call	wm_S
		call	wm_o
		call	wm_n
		call	wm_y
		call	wm_space
		call	wm_space
		call	wm_space
		call	wm_space
		call	wm_space
		call	wm_space
		return
;----------------------------------
w_wt4		call	wm_P
		call	wm_a
		call	wm_n
		call	wm_a
		call	wm_s
		call	wm_o
		call	wm_n
		call	wm_i
		call	wm_c
		return
;----------------------------------
w_wt5		call	wm_A
		call	wm_i
		call	wm_w
		call	wm_a
		call	wm_space
		call	wm_space
		call	wm_space
		call	wm_space
		call	wm_space
		call	wm_space
		return
;-------------------------------------------------------------------------------
w_status	movlw	B'10000000'
		call	send_comm

		movf	v_ir_byte9,0
		xorlw	H'FF'
		btfss	STATUS,Z
		goto	w_status2

		call	wm_S
		call	wm_e
		call	wm_l
		call	wm_e
		call	wm_c
		call	wm_t
		call	wm_e
		call	wm_d
		call	wm_space
		call	wm_c
		call	wm_o
		call	wm_d
		call	wm_e
		call	wm_dvojbodka
		call	wm_space

		movf	v_ir_code,0
		xorlw	D'001'
		btfsc	STATUS,Z
		call	w_wt1

		movf	v_ir_code,0
		xorlw	D'002'
		btfsc	STATUS,Z
		call	w_wt2

		movf	v_ir_code,0
		xorlw	D'003'
		btfsc	STATUS,Z
		call	w_wt3

		movf	v_ir_code,0
		xorlw	D'004'
		btfsc	STATUS,Z
		call	w_wt4

		movf	v_ir_code,0
		xorlw	D'005'
		btfsc	STATUS,Z
		call	w_wt5

;-----------------------------------------------
		movlw	B'11000000'
		call	send_comm

w_status2	movf	v_ir_byte1,0
		xorlw	H'FF'
		btfsc	STATUS,Z
		goto	w_ny

		movf	v_ir_byte1,0
		movwf	kbd_buffer
		call	write_hex
		call	wm_minus

		movf	v_ir_byte2,0
		movwf	kbd_buffer
		call	write_hex
		call	wm_minus

		movf	v_ir_byte3,0
		movwf	kbd_buffer
		call	write_hex
		call	wm_minus

		movf	v_ir_byte4,0
		movwf	kbd_buffer
		call	write_hex
		call	wm_minus

		movf	v_ir_byte5,0
		movwf	kbd_buffer
		call	write_hex
		call	wm_minus

		movf	v_ir_byte6,0
		movwf	kbd_buffer
		call	write_hex
		call	wm_minus

		movf	v_ir_byte7,0
		movwf	kbd_buffer
		call	write_hex
		call	wm_minus

		movf	v_ir_byte8,0
		movwf	kbd_buffer
		call	write_hex

		call	wm_space

		movf	v_ir_byte9,0
		xorlw	H'FF'
		btfss	STATUS,Z
		goto	w_status3

		return
;-------------------------------------------------------------------------------
w_ny		call	wm_minus
		call	wm_minus
		call	wm_space
		call	wm_space
		call	wm_space
		call	wm_space
		call	wm_space
		call	wm_space
		call	wm_space
		call	wm_space
		call	wm_space
		call	wm_space
		call	wm_space
		call	wm_space
		call	wm_space
		call	wm_space
		call	wm_space
		call	wm_space
		call	wm_space
		call	wm_space
		call	wm_space
		call	wm_space
		call	wm_space
		call	wm_space
		return
;-------------------------------------------------------------------------------
;-------------------------------------------------------------------------------
w_status3	movlw	B'11000000'
		call	send_comm


		movf	v_ir_byte9,0
		movwf	kbd_buffer
		call	write_hex
		call	wm_minus

		movf	v_ir_byte10,0
		movwf	kbd_buffer
		call	write_hex
		call	wm_minus

		movf	v_ir_byte11,0
		movwf	kbd_buffer
		call	write_hex
		call	wm_minus

		movf	v_ir_byte12,0
		movwf	kbd_buffer
		call	write_hex
		call	wm_minus

		movf	v_ir_byte13,0
		movwf	kbd_buffer
		call	write_hex
		call	wm_minus

		movf	v_ir_byte14,0
		movwf	kbd_buffer
		call	write_hex
		call	wm_minus

		movf	v_ir_byte15,0
		movwf	kbd_buffer
		call	write_hex
		call	wm_minus

		movf	v_ir_byte16,0
		movwf	kbd_buffer
		call	write_hex

		call	wm_space

		return

;-------------------------------------------------------------------------------
;-------------------------------------------------------------------------------
w_release	call	init2

		movlw	B'10000000'		;row: 1 / col: 0
		call	send_comm

		call	wm_I
		call	wm_R
		call	wm_space
		call	wm_a
		call	wm_n
		call	wm_a
		call	wm_l
		call	wm_y
		call	wm_s
		call	wm_e
		call	wm_r
		call	wm_space
		call	wm_minus
		call	wm_space
		call	wm_o
		call	wm_k
		call	wm_r
		call	wm_o
		call	wm_space
		call	wm_c
		call	wm_o
		call	wm_d
		call	wm_e


		movlw	B'11000000'		;row: 2 / col: 0
		call	send_comm

		call	wm_F
		call	wm_i
		call	wm_r
		call	wm_m
		call	wm_w
		call	wm_a
		call	wm_r
		call	wm_e
		call	wm_space
		call	wm_1
		call	wm_bodka
		call	wm_2
		call	wm_2
		call	wm_minus
		call	wm_1
		call	wm_4
		call	wm_bodka
		call	wm_0
		call	wm_1
		call	wm_bodka
		call	wm_2
		call	wm_0
		call	wm_1
		call	wm_5

		return
;-------------------------------------------------------------------------------
;-------------------------------------------------------------------------------

;-------------------------------------------------------------------------------
;-------------------------------------------------------------------------------
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;-------------------------------------------------------------------------------
;-------------------------------------------------------------------------------
;-------------------------------------------------------------------------------
write_bin7	movlw	'0'
		btfsc	kbd_buffer,D'007'
		movlw	'1'
		call	send_data

		movlw	'0'
		btfsc	kbd_buffer,D'006'
		movlw	'1'
		call	send_data

		movlw	'0'
		btfsc	kbd_buffer,D'005'
		movlw	'1'
		call	send_data

		movlw	'0'
		btfsc	kbd_buffer,D'004'
		movlw	'1'
		call	send_data

		movlw	'0'
		btfsc	kbd_buffer,D'003'
		movlw	'1'
		call	send_data

		movlw	'0'
		btfsc	kbd_buffer,D'002'
		movlw	'1'
		call	send_data

		movlw	'0'
		btfsc	kbd_buffer,D'001'
		movlw	'1'
		call	send_data

		call	wm_x

		return
;-------------------------------------------------------------------------------
;-------------------------------------------------------------------------------

;-------------------------------------------------------------------------------
write_bin	movlw	'0'
		btfsc	kbd_buffer,D'007'
		movlw	'1'
		call	send_data

		movlw	'0'
		btfsc	kbd_buffer,D'006'
		movlw	'1'
		call	send_data

		movlw	'0'
		btfsc	kbd_buffer,D'005'
		movlw	'1'
		call	send_data

		movlw	'0'
		btfsc	kbd_buffer,D'004'
		movlw	'1'
		call	send_data

		movlw	'0'
		btfsc	kbd_buffer,D'003'
		movlw	'1'
		call	send_data

		movlw	'0'
		btfsc	kbd_buffer,D'002'
		movlw	'1'
		call	send_data

		movlw	'0'
		btfsc	kbd_buffer,D'001'
		movlw	'1'
		call	send_data

		movlw	'0'
		btfsc	kbd_buffer,D'000'
		movlw	'1'
		call	send_data

		return

;-------------------------------------------------------------------------------
;-------------------------------------------------------------------------------
write_dec	movlw	'0'
		movwf	v_tst1
		movwf	v_tst2
		movwf	v_tst3

conv_next	movf	kbd_buffer,0
		btfsc	STATUS,Z
		goto	tst_conv_f

		incf	v_tst3,1
		movf	v_tst3,0
		xorlw	':'
		btfss	STATUS,Z
		goto	wdcn

		movlw	'0'
		movwf	v_tst3
		incf	v_tst2,1
		movf	v_tst2,0
		xorlw	':'
		btfss	STATUS,Z
		goto	wdcn

		movlw	'0'
		movwf	v_tst2
		incf	v_tst1,1

wdcn		decf	kbd_buffer,1
		goto	conv_next
;-------------------------------------------------
tst_conv_f	movf	v_tst1,0
;;;		call	send_data		;do not write first digit
		movf	v_tst2,0
		call	send_data
		movf	v_tst3,0
		call	send_data
		return
;-------------------------------------------------------------------------------
;-------------------------------------------------------------------------------
write_hex	nop

		movf	kbd_buffer,0
		andlw	B'11110000'
		xorlw	H'00'
		btfsc	STATUS,Z
		call	wm_0

		movf	kbd_buffer,0
		andlw	B'11110000'
		xorlw	H'10'
		btfsc	STATUS,Z
		call	wm_1

		movf	kbd_buffer,0
		andlw	B'11110000'
		xorlw	H'20'
		btfsc	STATUS,Z
		call	wm_2

		movf	kbd_buffer,0
		andlw	B'11110000'
		xorlw	H'30'
		btfsc	STATUS,Z
		call	wm_3

		movf	kbd_buffer,0
		andlw	B'11110000'
		xorlw	H'40'
		btfsc	STATUS,Z
		call	wm_4

		movf	kbd_buffer,0
		andlw	B'11110000'
		xorlw	H'50'
		btfsc	STATUS,Z
		call	wm_5

		movf	kbd_buffer,0
		andlw	B'11110000'
		xorlw	H'60'
		btfsc	STATUS,Z
		call	wm_6

		movf	kbd_buffer,0
		andlw	B'11110000'
		xorlw	H'70'
		btfsc	STATUS,Z
		call	wm_7

		movf	kbd_buffer,0
		andlw	B'11110000'
		xorlw	H'80'
		btfsc	STATUS,Z
		call	wm_8

		movf	kbd_buffer,0
		andlw	B'11110000'
		xorlw	H'90'
		btfsc	STATUS,Z
		call	wm_9

		movf	kbd_buffer,0
		andlw	B'11110000'
		xorlw	H'A0'
		btfsc	STATUS,Z
		call	wm_A

		movf	kbd_buffer,0
		andlw	B'11110000'
		xorlw	H'B0'
		btfsc	STATUS,Z
		call	wm_B

		movf	kbd_buffer,0
		andlw	B'11110000'
		xorlw	H'C0'
		btfsc	STATUS,Z
		call	wm_C

		movf	kbd_buffer,0
		andlw	B'11110000'
		xorlw	H'D0'
		btfsc	STATUS,Z
		call	wm_D

		movf	kbd_buffer,0
		andlw	B'11110000'
		xorlw	H'E0'
		btfsc	STATUS,Z
		call	wm_E

		movf	kbd_buffer,0
		andlw	B'11110000'
		xorlw	H'F0'
		btfsc	STATUS,Z
		call	wm_F
;
		movf	kbd_buffer,0
		andlw	B'00001111'
		xorlw	H'00'
		btfsc	STATUS,Z
		goto	wm_0

		movf	kbd_buffer,0
		andlw	B'00001111'
		xorlw	H'01'
		btfsc	STATUS,Z
		goto	wm_1

		movf	kbd_buffer,0
		andlw	B'00001111'
		xorlw	H'02'
		btfsc	STATUS,Z
		goto	wm_2

		movf	kbd_buffer,0
		andlw	B'00001111'
		xorlw	H'03'
		btfsc	STATUS,Z
		goto	wm_3

		movf	kbd_buffer,0
		andlw	B'00001111'
		xorlw	H'04'
		btfsc	STATUS,Z
		goto	wm_4

		movf	kbd_buffer,0
		andlw	B'00001111'
		xorlw	H'05'
		btfsc	STATUS,Z
		goto	wm_5

		movf	kbd_buffer,0
		andlw	B'00001111'
		xorlw	H'06'
		btfsc	STATUS,Z
		goto	wm_6

		movf	kbd_buffer,0
		andlw	B'00001111'
		xorlw	H'07'
		btfsc	STATUS,Z
		goto	wm_7

		movf	kbd_buffer,0
		andlw	B'00001111'
		xorlw	H'08'
		btfsc	STATUS,Z
		goto	wm_8

		movf	kbd_buffer,0
		andlw	B'00001111'
		xorlw	H'09'
		btfsc	STATUS,Z
		goto	wm_9

		movf	kbd_buffer,0
		andlw	B'00001111'
		xorlw	H'0A'
		btfsc	STATUS,Z
		goto	wm_A

		movf	kbd_buffer,0
		andlw	B'00001111'
		xorlw	H'0B'
		btfsc	STATUS,Z
		goto	wm_B

		movf	kbd_buffer,0
		andlw	B'00001111'
		xorlw	H'0C'
		btfsc	STATUS,Z
		goto	wm_C

		movf	kbd_buffer,0
		andlw	B'00001111'
		xorlw	H'0D'
		btfsc	STATUS,Z
		goto	wm_D

		movf	kbd_buffer,0
		andlw	B'00001111'
		xorlw	H'0E'
		btfsc	STATUS,Z
		goto	wm_E

		movf	kbd_buffer,0
		andlw	B'00001111'
		xorlw	H'0F'
		btfsc	STATUS,Z
		goto	wm_F


		return

;-----------------------------------------------------------------------------------------
;-----------------------------------------------------------------------------------------
;-----------------------------------------------------------------------------------------
;-----------------------------------------------------------------------------------------
;xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx


		END