
;**************************************************************************
;*                    IR Analyzer                                         *
;**************************************************************************
;**************************************************************************
;*          v 1.14 - 31.12.2014                                           *
;*                                                                        *
;**************************************************************************
;*          Hardw.: PIC12F1822 used                                       *
;*          OSC.......: Internal osc. used     POWER.....:   5V extern.   *
;**************************************************************************


 INCLUDE "P12F1822.INC"



_U001			     EQU     B'00100110000100'		;see Page 48 of Datasheet
_U002			     EQU     B'01100011111111'		;see Page 49 of Datasheet


	__CONFIG	_CONFIG1, _U001
	__CONFIG	_CONFIG2, _U002



;==========================================================================
;       Variable Definition
;==========================================================================
TIMER1		EQU	H'20'		;delay routine
TIMER2		EQU	H'21'		; "	"	"

v_ttt		EQU	H'22'
v_cnt1		EQU	H'23'
v_cnt2		EQU	H'24'
kbd_buffer	EQU	H'25'
v_tst1		EQU	H'26'
v_tst2		EQU	H'27'
v_tst3		EQU	H'28'
disp_pos	EQU	H'29'

v_ir_bit	EQU	H'2A'
v_itimer1	EQU	H'2B'
v_itimer2	EQU	H'2C'
v_ir_tmp	EQU	H'2D'
v_ir_new	EQU	H'2E'
v_ir_byte	EQU	H'2F'

v_ir_byte1	EQU	H'30'
v_ir_byte2	EQU	H'31'
v_ir_byte3	EQU	H'32'
v_ir_byte4	EQU	H'33'
v_ir_byte5	EQU	H'34'
v_ir_byte6	EQU	H'35'
v_ir_byte7	EQU	H'36'
v_ir_byte8	EQU	H'37'

v_ir_byte1_s	EQU	H'38'
v_ir_byte2_s	EQU	H'39'
v_ir_byte3_s	EQU	H'3A'
v_ir_byte4_s	EQU	H'3B'
v_ir_byte5_s	EQU	H'3C'
v_ir_byte6_s	EQU	H'3D'
v_ir_byte7_s	EQU	H'3E'
v_ir_byte8_s	EQU	H'3F'

v_ph1d_m	EQU	H'40'
v_ph1d_l	EQU	H'41'
v_ph2d_m	EQU	H'42'
v_ph2d_l	EQU	H'43'

;--------------------------------------------------------------------------
v_ir_code	EQU	H'44'		;selected IR-code (pulse width)

	;01 = Philips RC6
	;02 = Sharp
;--------------------------------------------------------------------------

v_stage		EQU	H'45'

;--------------------------------------------------------------------------
;==========================================================================

		ORG	0		;Reset vector address
		GOTO	init_main	;goto RESET routine when boot.

		ORG	4
		goto	ir_main		;interrupt service routine

;----------------------------------------------------------------------------
;----------------------------------------------------------------------------
;
;	       **********************************
;              **  RESET :  main boot routine  **
;              **********************************
;
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
; initialization
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------

init_main	nop

		clrf	BSR
;
		clrf	INTCON		;disable interrupts

		banksel	OPTION_REG	;BANK 1
		movlw	B'00111111'
		movwf	OPTION_REG	;see datasheet Page 176

		movlw	H'03'
		movwf	BSR		;select BANK 3
		clrf	ANSELA

		movlw	H'01'
		movwf	BSR		;select BANK 1
		movlw	B'11011100'
		movwf	TRISA

		movf	OSCCON,0
		andlw	B'11111111'
		iorlw	B'01111000'
		movwf	OSCCON		;set 16MHz internal clock (bit 6:3 = 1111) - Datasheet Page 68

		movlw	H'04'
		movwf	BSR		;select BANK 4
		movlw	B'00100100'
		movwf	WPUA		;enable weak pull-ups

		clrf	BSR		;select BANK 0

		clrf	PORTA
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------

;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
;set-up interrupt in INT-pin (RA2)
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------

		clrf	INTCON

;bit INTEDG (bit 6) of OPTION_REG needs to be cleared - is defined above

		bsf	INTCON,INTE			;Enables the INT external interrupt
		bsf	INTCON,GIE			;Enables all active interrupts
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------

		goto	main_prg


;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;------------------------------------------------------------------------------
;                      Interrupt service routine
;------------------------------------------------------------------------------
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX

ir_main		nop
		btfss	INTCON,INTF		;unkown event -- return
		goto	leave_ir

		bcf	INTCON,D'007'		;disable interrupts
		movf	PORTA,0
		bcf	INTCON,INTF

		btfsc	v_ir_new,D'000'
		goto	leave_ir		;last value not processed yet

;-----------------------------------------------------------------------------
;Interrupt business logic starts here....
;-----------------------------------------------------------------------------

		bsf	PORTA,D'000'		;switch the LED on - we got signal...
		btfsc	PORTA,D'002'
		goto	ir_cancel		;we expect level 0 now...

		call	ir_get_byte		;receive 1st byte
		movf	v_ir_byte,0
		movwf	v_ir_byte1_s

		call	ir_get_byte		;receive 2nd byte
		movf	v_ir_byte,0
		movwf	v_ir_byte2_s

		call	ir_get_byte		;receive 3rd byte
		movf	v_ir_byte,0
		movwf	v_ir_byte3_s

		call	ir_get_byte		;receive 4th byte
		movf	v_ir_byte,0
		movwf	v_ir_byte4_s

		call	ir_get_byte		;receive 5th byte
		movf	v_ir_byte,0
		movwf	v_ir_byte5_s

		call	ir_get_byte		;receive 6th byte
		movf	v_ir_byte,0
		movwf	v_ir_byte6_s

		call	ir_get_byte		;receive 7th byte
		movf	v_ir_byte,0
		movwf	v_ir_byte7_s

		call	ir_get_byte		;receive 8th byte
		movf	v_ir_byte,0
		movwf	v_ir_byte8_s

		movf	v_ir_byte1_s,0
;;;;		xorlw	H'14'
		xorlw	H'03'
		btfss	STATUS,Z
		nop
;;;;;;		goto	leave_ir		;not valid code received (first byte is always 14h) --> exit

		movf	v_ir_byte1_s,0
		movwf	v_ir_byte1
		movf	v_ir_byte2_s,0
		movwf	v_ir_byte2
		movf	v_ir_byte3_s,0
		movwf	v_ir_byte3
		movf	v_ir_byte4_s,0
		movwf	v_ir_byte4
		movf	v_ir_byte5_s,0
		movwf	v_ir_byte5
		movf	v_ir_byte6_s,0
		movwf	v_ir_byte6
		movf	v_ir_byte7_s,0
		movwf	v_ir_byte7
		movf	v_ir_byte8_s,0
		movwf	v_ir_byte8

		clrf	v_ir_new		;store the code
		bsf	v_ir_new,D'000'		;set flag - new character received

;----------------------------------------
ir_cancel	bcf	PORTA,D'000'		;switch the LED off
;-----------------------------------------------------------------------------
;Interrupt business logic ends here....
;-----------------------------------------------------------------------------
leave_ir	nop
		bcf	PORTA,D'000'		;switch the LED off
		bsf	INTCON,INTE
		bsf	INTCON,D'007'
		RETFIE				;return from interrupt with restore of STAUS, WREG and BSR
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
ir_get_byte	clrf	v_ir_byte
		call	ir_get_bit
		btfsc	v_ir_bit,D'000'
		bsf	v_ir_byte,D'007'

		call	ir_get_bit
		btfsc	v_ir_bit,D'000'
		bsf	v_ir_byte,D'006'

		call	ir_get_bit
		btfsc	v_ir_bit,D'000'
		bsf	v_ir_byte,D'005'

		call	ir_get_bit
		btfsc	v_ir_bit,D'000'
		bsf	v_ir_byte,D'004'

		call	ir_get_bit
		btfsc	v_ir_bit,D'000'
		bsf	v_ir_byte,D'003'

		call	ir_get_bit
		btfsc	v_ir_bit,D'000'
		bsf	v_ir_byte,D'002'

		call	ir_get_bit
		btfsc	v_ir_bit,D'000'
		bsf	v_ir_byte,D'001'

		call	ir_get_bit
		btfsc	v_ir_bit,D'000'
		bsf	v_ir_byte,D'000'
		return
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
ir_get_bit	clrf	v_ir_bit
		call	wait_Ph1
		btfsc	PORTA,D'002'
		bsf	v_ir_bit,D'000'
		call	wait_Ph2
		return
;------------------------------------------------------------------------------
ir_get_bit2	clrf	v_ir_bit
		call	wait300us
		btfsc	PORTA,D'002'
		bsf	v_ir_bit,D'000'
		call	wait460us
		return
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
; interrupt delay routines
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
d350		MOVLW   D'255'
		MOVWF   v_itimer2
iDEL_LOOP1	MOVLW   D'255'
		MOVWF   v_itimer1

iDEL_LOOP2	DECFSZ  v_itimer1,F
		GOTO    iDEL_LOOP2
		DECFSZ  v_itimer2,F
		GOTO    iDEL_LOOP1
		return
;----------------------------------------------------------------------------
;----------------------------------------------------------------------------
;Phase 1 wait routine
;----------------------------------------------------------------------------
;----------------------------------------------------------------------------
wait_Ph1	movf	v_ph1d_m,0
		movwf	v_itimer2
iw2_l2		movf	v_ph1d_l,0
		movwf	v_itimer1

iw2_l1		decfsz	v_itimer1,1		;0,25us
		goto	iw2_l1			;0,50us

		decfsz	v_itimer2,1
		goto	iw2_l2
		return

;----------------------------------------------------------------------------
;----------------------------------------------------------------------------
;Phase 2 wait routine
;----------------------------------------------------------------------------
;----------------------------------------------------------------------------
wait_Ph2	movf	v_ph2d_m,0
		movwf	v_itimer2
iw28_l2		movf	v_ph2d_l,0
		movwf	v_itimer1

iw28_l1		decfsz	v_itimer1,1		;0,25us
		goto	iw28_l1			;0,50us (16MHz clock)

		decfsz	v_itimer2,1
		goto	iw28_l2
		return

;----------------------------------------------------------------------------
;----------------------------------------------------------------------------
;----------------------------------------------------------------------------
wait300us	movlw	D'002'
		movwf	v_itimer2
iw3_l2		movlw	D'210'
		movwf	v_itimer1

iw3_l1		decfsz	v_itimer1,1		;0,5us / 2
		goto	iw3_l1			;1,0us / 2

		decfsz	v_itimer2,1
		goto	iw3_l2
		return

;----------------------------------------------------------------------------
wait460us	movlw	D'004'
		movwf	v_itimer2
iw4_l2		movlw	D'155'
		movwf	v_itimer1

iw4_l1		decfsz	v_itimer1,1		;0,5us / 2
		goto	iw4_l1			;1,0us / 2

		decfsz	v_itimer2,1
		goto	iw4_l2
		return

;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;------------------------------------------------------------------------------
;                      Waiting times definitions
;------------------------------------------------------------------------------
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------

;----------------------------------------------------
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;----------------------------------------------------
;delay values for Philips code RC6
;PIC Clock = 16MHz
;in total: 444us
;
;Phase 1 in total ~ 1 * 245 * 0,75us = 183,75 us
;Phase 2 in total ~ 2 * 169 * 0,75us = 253,5 us
;----------------------------------------------------
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;----------------------------------------------------

wt1_philips1	movlw	D'001'
		movwf	v_ph1d_m
		movlw	D'245'
		movwf	v_ph1d_l

		movlw	D'002'
		movwf	v_ph2d_m
		movlw	D'169'
		movwf	v_ph2d_l

		return

;----------------------------------------------------
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;----------------------------------------------------
;delay values for Sharp
;PIC Clock = 16MHz
;in total: 300us
;
;Phase 1 in total ~ 1 * 133 * 0,75us = 99,75 us
;Phase 2 in total ~ 2 * 133 * 0,75us = 199,5 us
;----------------------------------------------------
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;----------------------------------------------------

wt2_sharp1	movlw	D'001'
		movwf	v_ph1d_m
		movlw	D'133'
		movwf	v_ph1d_l

		movlw	D'002'
		movwf	v_ph2d_m
		movlw	D'133'
		movwf	v_ph2d_l

		return

;----------------------------------------------------
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;----------------------------------------------------
;delay values for Sony
;PIC Clock = 16MHz
;in total: 600us
;
;Phase 1 in total ~ 2 * 133 * 0,75us = 99,75 us
;Phase 2 in total ~ 4 * 133 * 0,75us = 199,5 us
;----------------------------------------------------
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;----------------------------------------------------

wt3_sony1	movlw	D'002'
		movwf	v_ph1d_m
		movlw	D'133'
		movwf	v_ph1d_l

;in total ~ 133 * 2 * 0,75us = 199,5 us

		movlw	D'004'
		movwf	v_ph2d_m
		movlw	D'133'
		movwf	v_ph2d_l

		return


;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
select_times	movf	v_ir_code,0
		xorlw	D'001'
		btfsc	STATUS,Z
		goto	wt1_philips1			;set-up times for Philips RC6 code

		movf	v_ir_code,0
		xorlw	D'002'
		btfsc	STATUS,Z
		goto	wt2_sharp1			;set-up times for "Sharp code"

		movf	v_ir_code,0
		xorlw	D'003'
		btfsc	STATUS,Z
		goto	wt3_sony1			;sony


		goto	wt1_philips1			;default...
		return


;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;------------------------------------------------------------------------------
;                      Main Loop
;------------------------------------------------------------------------------
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------

main_prg	movlw	H'FF'
		movwf	v_ir_byte1
		movwf	v_ir_new


		bcf	PORTA,D'005'
		clrf	v_stage

		movlw	D'001'
		movwf	v_ir_code		;select Philips RC6 IR-code
		call	select_times

		call	start_blink

		clrf	v_cnt1
		clrf	v_cnt2


main_prg2	nop
		clrf	v_ir_new

mp_wait		call	LEDs
		btfss	v_ir_new,D'000'
		goto	mp_wait

;now we received something
		call	execute_comm

		goto	main_prg2
;------------------------------------------------------------------------------
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;------------------------------------------------------------------------------

LEDs		incf	v_cnt2,1
		btfss	STATUS,Z
		return

		incf	v_cnt1,1
		btfss	v_cnt1,D'002'
		goto	LEDs_off		;switch both LEDs off

;based on stage - switch red or green LED on
		btfss	v_stage,D'000'
		goto	LED_i_off
		goto	LED_i_on
;----------------------------------------
LED_i_off	bcf	PORTA,D'000'
		bsf	PORTA,D'001'
		return
;----------------------------------------
LED_i_on	bsf	PORTA,D'000'
		bcf	PORTA,D'001'
		return
;----------------------------------------
LEDs_off	bcf	PORTA,D'000'
		bcf	PORTA,D'001'
		return
;------------------------------------------------------------------------------
start_blink	nop

		bsf	PORTA,D'000'
		call	dr2
		bcf	PORTA,D'000'
		call	dr2
		bsf	PORTA,D'000'
		call	dr2
		bcf	PORTA,D'000'
		call	dr2
		bsf	PORTA,D'000'
		call	dr2
		bcf	PORTA,D'000'
		call	dr2
		bsf	PORTA,D'000'
		call	dr2
		bcf	PORTA,D'000'
		call	dr2

		return
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
;analyze the received IR-code and execute if there is something to be executed
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
execute_comm	nop

		movf	v_ir_byte1,0
		xorlw	H'03'
		btfss	STATUS,Z
		return				;not a valid code

		movf	v_ir_byte2,0
		xorlw	H'6A'
		btfss	STATUS,Z
		return				;not a valid code

;ignore byte 3 (can be 3A or CA)

		movf	v_ir_byte4,0
		xorlw	H'AA'
		btfss	STATUS,Z
		return				;not a valid code

		movf	v_ir_byte5,0
		xorlw	H'A9'
		btfss	STATUS,Z
		return				;not a valid code

		movf	v_ir_byte6,0
		xorlw	H'65'
		btfss	STATUS,Z
		return				;not a valid code

		movf	v_ir_byte8,0
		xorlw	H'FF'
		btfss	STATUS,Z
		return				;not a valid code

		movf	v_ir_byte7,0
		xorlw	H'6F'
		btfsc	STATUS,Z
		goto	sw_on			;"green button" has been pressed

		movf	v_ir_byte7,0
		xorlw	H'9F'
		btfsc	STATUS,Z
		goto	sw_off			;"red button" has been pressed

		return

;----------------------------------------------
sw_on		bsf	PORTA,D'005'
		bsf	v_stage,D'000'
		return
;----------------------------------------------
sw_off		bcf	PORTA,D'005'
		bcf	v_stage,D'000'
		return

;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------

;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
; delay routines
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
d2		MOVLW   D'255'
		MOVWF   TIMER2
DEL_LOOP1	MOVLW   D'255'
		MOVWF   TIMER1

DEL_LOOP2	DECFSZ  TIMER1,F
		GOTO    DEL_LOOP2
		DECFSZ  TIMER2,F
		GOTO    DEL_LOOP1
		return
;------------------------------------------------------------------------------
d2c		MOVLW   D'090'
		MOVWF   TIMER2
cDEL_LOOP1	MOVLW   D'255'
		MOVWF   TIMER1

cDEL_LOOP2	DECFSZ  TIMER1,F
		GOTO    cDEL_LOOP2
		DECFSZ  TIMER2,F
		GOTO    cDEL_LOOP1
		return
;------------------------------------------------------------------------------

;------------------------------------------------------------------------------
;------------------------------------------------------------------------------

;------------------------------------
dr2		call	d2
		call	d2
		call	d2
		return
;------------------------------------
dr4		call	dr2
		call	dr2
		call	dr2
		return


;-----------------------------------------------------------------------------------------
;-----------------------------------------------------------------------------------------
;-----------------------------------------------------------------------------------------
;-----------------------------------------------------------------------------------------
;xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx


		END